/**
 * @file md5.h
 * @brief Public-Domain MD5 interface description, massaged for POSIX-
 * and ScalES-PPM-conformance
 *
 * @copyright Copyright  (C)  2012  Thomas Jahns <jahns@dkrz.de>
 *
 * @version 1.0
 * @author Thomas Jahns <jahns@dkrz.de>
 */
/*
 * Keywords:
 * Maintainer: Thomas Jahns <jahns@dkrz.de>
 * URL: https://www.dkrz.de/redmine/projects/scales-ppm
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are  permitted provided that the following conditions are
 * met:
 *
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * Neither the name of the DKRZ GmbH nor the names of its contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*	$OpenBSD: md5.h,v 1.1.2.1 2004/06/05 23:12:36 niklas Exp $	*/

/*
 * Adapted by Thomas Jahns to fit ScalES-PPM coding conventions
 */
/*
 * This code implements the MD5 message-digest algorithm.
 * The algorithm is due to Ron Rivest.  This code was
 * written by Colin Plumb in 1993, no copyright is claimed.
 * This code is in the public domain; do with it what you wish.
 *
 * Equivalent code is available from RSA Data Security, Inc.
 * This code has been tested against that, and is equivalent,
 * except that you don't need to include two pages of legalese
 * with every copy.
 */

#ifndef PPM_MD5_H
#define PPM_MD5_H

#include <inttypes.h>
#include <stdlib.h>

#include "core/ppm_visibility.h"

enum {
  PPM_MD5_BLOCK_LENGTH = 64,
  PPM_MD5_DIGEST_LENGTH = 16,
};

typedef struct MD5Context {
  uint32_t state[4];                    /* state */
  uint64_t count;                       /* number of bits, mod 2^64 */
  uint8_t buffer[PPM_MD5_BLOCK_LENGTH];     /* input buffer */
} PPM_MD5_CTX;

void PPM_DSO_INTERNAL
PPM_MD5Init(PPM_MD5_CTX *);
void PPM_DSO_INTERNAL
PPM_MD5Update(PPM_MD5_CTX *, const uint8_t *, size_t);
void PPM_DSO_INTERNAL
PPM_MD5Final(uint8_t [PPM_MD5_DIGEST_LENGTH], PPM_MD5_CTX *);

#endif /* PPM_MD5_H */
/*
 * Local Variables:
 * license-project-url: "https://www.dkrz.de/redmine/projects/scales-ppm"
 * license-markup: "doxygen"
 * license-default: "bsd"
 * End:
 */
